/* Beneath Enemy Lines Submarine Combat Prototype 1
 * An exercise in multiple naval objects: their creating and destruction.
 * Combat Prototype 1
 * Copyright (c) 2001 David Young

 * Having established the user's submarine in the Control Prototype,
 work now focuses on creating multiple submarines as well as other
 sea objects.  With other boats in the water, the user--and the user's
 adversaries--have incentive to launch torpedoes in full-scale
 submarine warfare.  Prototype 1 seeks to create the submarines and
 torpedoes in one stroke by splitting the original Sub10 class
 into the Benel superclass, creating graphics objects from a separate
 class as well as making naval objects from a vehicles class.
 The prototype will allow the user to fire torpedoes toward
 a specified coordinate.
 */

import java.awt.*;
import javax.swing.*;
import javax.swing.Timer;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;
import java.text.*;

/** <code>Benel</code> is Beneath Enemy Line's superclass. 
    Intricately tied with the user's own sub, Benel creates a gui
    that calls a JTabbed Pane to house all the sub's controls and displays.
    Benel also creates ArrayLists to hold all the other subs, 
    torpedoes, and other naval objects.

    "Own" refers to user's submaine and all other elements on user's
    side.

    "Others" refers to all other submarines.
*/
public class Benel extends JFrame implements ActionListener, ChangeListener, FocusListener {
    private Timer timer;

    private ScopePanel scope = new ScopePanel();;
    private FireControlPanel fireControl;
    private JLabel positionLbl = new JLabel("Position (mi):");
    private JLabel speedLbl = new JLabel("Speed (knots):");
    private JLabel depthLbl = new JLabel("Depth (feet):");
    private JLabel bearingLbl = new JLabel("Bearing (deg):");
    private JLabel seaDepthLbl = new JLabel("Sea Depth (feet):");
    private JLabel testLbl = new JLabel("Test");
    /* whenever move mouse over JTextField, resizes if don't
       specify absolute size */
    private JTextField positionDisp = new JTextField(12);
    private JTextField position2Disp = new JTextField(12);
    private JTextField speedDisp = new JTextField(6);
    private JTextField speed2Disp = new JTextField(6);
    private JTextField depthDisp = new JTextField(6);
    private JTextField depth2Disp = new JTextField(6);
    private JTextField bearingDisp = new JTextField(6);
    private JTextField bearing2Disp = new JTextField(6);
    private JTextField seaDepthDisp = new JTextField(6);
    private JTextField testDisp = new JTextField(6);
    private JSlider speed = new JSlider(JSlider.VERTICAL, 0, 30, 0);
    private JSlider depth = new JSlider(JSlider.VERTICAL, 0, 2000, 0);
    private JSlider bearing = new JSlider(JSlider.VERTICAL, 0, 360, 0);
    private JButton move = new JButton("Declare Move");

    private static Sub own;
    // "others" could be on same side or opponent as of now
    private static ArrayList others = new ArrayList();
    private static ArrayList torpedoes = new ArrayList();

    public static final double DT = .01; // can adjust for accuracy?  If too small, program seems to fail; may want to make private somehow
    public static final double DP = .005; // extra position factor
    // may want to make origin int since only refers to position on comp
    private static Point2D.Double origin = new Point2D.Double(0, 0);
    // private static Point2D.Double subCoordRecen = new Point2D.Double(origin.x, origin.y);

    private String htFldName = "htFld.bmp";
    private int[] htFldRGB = new int[230400]; // 320 x 240 pixel picture
    private static int[] htFld = new int[76800]; // Blue values of 320 x 240 pixel picture

    /** Creates a JTabbedPane to house the submarine's various controls
	and displays. Also, creates the submarine's "bridge", though
	this task may be relegated to a separate class in the future.
     */
    public Benel() {
	super("Beneath Enemy Lines");
	setSize(900, 500);
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

	JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);

	GridBagLayout gridBag = new GridBagLayout();
	GridBagConstraints constraints = new GridBagConstraints(); // currently 100x110

	JPanel pane = new JPanel();
	pane.setLayout(gridBag);

	// Scope Panel
	/* Helpful to set all cells on first line, breaking up larger 
	 * cells and having larger components span several smaller ones instead
	 */
	buildConstraints(constraints, 0, 0, 2, 4, 100, 100); // height weighted at 10 per normal line
	constraints.fill = GridBagConstraints.BOTH;
	constraints.anchor = GridBagConstraints.CENTER;
	gridBag.setConstraints(scope, constraints);
	pane.add(scope);

	//Current depth lbl
	buildConstraints(constraints, 2, 0, 1, 1, 50, 100);
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.EAST;
	gridBag.setConstraints(depthLbl, constraints);
	pane.add(depthLbl);

	// Current depth text field
	depthDisp.setText(ownExists() ? (own.getDepth() + "") : "0");
	depthDisp.setEditable(false);
	buildConstraints(constraints, 2, 1, 1, 1, 0, 100);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(depthDisp, constraints);
	pane.add(depthDisp);

	// Depth slider
	buildConstraints(constraints, 2, 2, 1, 1, 0, 100);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(depth, constraints);
	depth.setMajorTickSpacing(500);
	depth.setMinorTickSpacing(100);
	depth.setPaintTicks(true);
	depth.setPaintLabels(true);
	depth.addChangeListener(this);
	pane.add(depth);

	// New depth text field
	depth2Disp.setText(ownExists() ? (own.getBearing() + "") : "0");
	depth2Disp.setEditable(true);
	depth2Disp.addFocusListener(this);
	buildConstraints(constraints, 2, 3, 1, 1, 0, 100);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(depth2Disp, constraints);
	pane.add(depth2Disp);

	// Current speed lbl
	buildConstraints(constraints, 3, 0, 1, 1, 50, 100);
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.EAST;
	gridBag.setConstraints(speedLbl, constraints);
	pane.add(speedLbl);

	// Current speed text field
	speedDisp.setText(ownExists() ? (own.getSpeed() + "") : "0");
	speedDisp.setEditable(false);
	buildConstraints(constraints, 3, 1, 1, 1, 0, 100);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(speedDisp, constraints);
	pane.add(speedDisp);

	// Speed slider
	buildConstraints(constraints, 3, 2, 1, 1, 0, 100);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(speed, constraints);
	speed.setMajorTickSpacing(5);
	speed.setMinorTickSpacing(1);
	speed.setPaintTicks(true);
	speed.setPaintLabels(true);
	speed.addChangeListener(this);
	pane.add(speed);

	// New speed text field
	speed2Disp.setText(ownExists() ? (own.getBearing() + "") : "0");
	speed2Disp.setEditable(true);
	speed2Disp.addFocusListener(this);
	buildConstraints(constraints, 3, 3, 1, 1, 0, 100);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(speed2Disp, constraints);
	pane.add(speed2Disp);

	// Current bearing lbl
	buildConstraints(constraints, 4, 0, 1, 1, 50, 100);
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.EAST;
	gridBag.setConstraints(bearingLbl, constraints);
	pane.add(bearingLbl);

	// Current bearing text field
	bearingDisp.setText(ownExists() ? (own.getBearing() + "") : "0");
	bearingDisp.setEditable(false);
	buildConstraints(constraints, 4, 1, 1, 1, 0, 100);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(bearingDisp, constraints);
	pane.add(bearingDisp);

	// Bearing slider
	buildConstraints(constraints, 4, 2, 1, 1, 0, 100);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(bearing, constraints);
	bearing.setMajorTickSpacing(60);
	bearing.setMinorTickSpacing(10);
	bearing.setPaintTicks(true);
	bearing.setPaintLabels(true);
	bearing.addChangeListener(this);
	pane.add(bearing);

	// New bearing text field
	bearing2Disp.setText(ownExists() ? (own.getBearing() + "") : "0");
	bearing2Disp.setEditable(true);
	bearing2Disp.addFocusListener(this);
	buildConstraints(constraints, 4, 3, 1, 1, 0, 100);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(bearing2Disp, constraints);
	pane.add(bearing2Disp);

	// Current position lbl
	buildConstraints(constraints, 0, 4, 1, 1, 100, 0);
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.EAST;
	gridBag.setConstraints(positionLbl, constraints);
	pane.add(positionLbl);

	// Current position text field
	positionDisp.setText(ownExists() ? ("( " + own.getCoord().x + ", " + own.getCoord().y + ")") : "(0, 0)");
	positionDisp.setEditable(false);
	buildConstraints(constraints, 1, 4, 1, 1, 0, 0);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(positionDisp, constraints);
	pane.add(positionDisp);

	// Move button
	move.addActionListener(this);
	move.setActionCommand("Move");
	buildConstraints(constraints, 2, 4, 3, 1, 0, 100);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	constraints.anchor = GridBagConstraints.CENTER;
	gridBag.setConstraints(move, constraints);
	pane.add(move);

	// Sea depth lbl
	buildConstraints(constraints, 0, 5, 1, 1, 100, 0);
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.EAST;
	gridBag.setConstraints(seaDepthLbl, constraints);
	pane.add(seaDepthLbl);

	// Sea depth text field
	seaDepthDisp.setText("");
	seaDepthDisp.setEditable(false);
	buildConstraints(constraints, 1, 5, 1, 1, 0, 0);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(seaDepthDisp, constraints);
	pane.add(seaDepthDisp);

	// Test lbl
	buildConstraints(constraints, 2, 5, 1, 1, 0, 100);
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.EAST;
	gridBag.setConstraints(testLbl, constraints);
	pane.add(testLbl);

	// Test text field
	/* See part of a text field on upper left corner if change 
	 * buildConstraints to cover 3 cells */
	testDisp.setText("");
	testDisp.setEditable(false);
	buildConstraints(constraints, 3, 5, 1, 1, 0, 100);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	gridBag.setConstraints(testDisp, constraints);
	pane.add(testDisp);

	tabbedPane.addTab("Command Workstation", null, pane, "Captian's orders");
	tabbedPane.setSelectedIndex(0);

	fireControl = new FireControlPanel();
	tabbedPane.addTab("Fire Control", null, fireControl, "Weapons delivery");

	// if use setContentPane(tabbedPane) instead, get misplaced paintings
	Container contentPane = getContentPane();
	// otherwise, will default to BorderLayout
	getContentPane().setLayout(new GridLayout(1, 1));
	contentPane.add(tabbedPane);

	try {
	    loadHtFld(htFldName);
	} catch(IOException e) {
	    System.out.println("Error: File not found.");
	}

	timer = new Timer(100, this);
	timer.start();
    }


    /* Class variable accessors
     */
    /** The number of other submarines.
	@return elements in <code>others</code>
    */
    public static int getNumOthers() {
	return others.size();
    }

    /** The number of torpedoes.
	@return elements in <code>torpedoes</code>
    */
    public static int getNumTorpedoes() {
	return torpedoes.size();
    }

    /** The re-defined origin, set at the center of the map in ScopePanel().
@return clone of origin
    */
    public static Point2D.Double getOrigin() {
	return (Point2D.Double)origin.clone();
    }

    // don't need to make clone b/c Vehicle objects already make them
    /** Own submarine's coordinates.
@return own coordinates
    */
    public static Point2D.Double getOwnCoord() {
	return own.getCoord();
    }

    /** Coordinates of a given submarine other than one's own submarine.
	@param i submarine's index
	@return coordinates of submarine <code>i</code>
    */
    public static Point2D.Double getOthersCoord(int i) {
	Sub s = (Sub)others.get(i);
	return s.getCoord();
    }

    /** Coordinates of a given torpedo.
	@param i torpedo's index
	@return coordinates of torpedo <code>i</code>
    */
    public static Point2D.Double getTorpedoCoord(int i) {
	Torpedo t = (Torpedo)torpedoes.get(i);
	return t.getCoord();
    }

    /** Ownership of a given submarine other relative to one's own ownership.
	@param i submarine's index
	@return ownership of submarine <code>i</code>
    */
    public static String getOthersOwnership(int i) {
	Sub s = (Sub)others.get(i);
	return s.getOwnership();
    }

    /** Ownership of a given torpedo relative to one's own ownership.
	@param i torpedo's index; can be own torpedo
	@return ownership of torpedo <code>i</code>
    */
    public static String getTorpedoesOwnership(int i) {
	Torpedo t = (Torpedo)torpedoes.get(i);
	return t.getOwnership();
    }

    /** The sea depth height field, created from the map 
	<code>htFld.bmp</code>.
	@return htFld
    */
    public static int[] getHtFld() {
	return htFld;
    }


    /* Class variable mutators
     */
    public static void setOrigin(double x, double y) {
	origin.x = x;
	origin.y = y;
    }

    public static void setOwn(Sub s) {
	own = s;
    }

    /* Extra mutators
     */



    public static void main(String[] args) {
	JFrame prgm = new Benel();
	ExitWindow exit = new ExitWindow();
	prgm.addWindowListener(exit);
	prgm.show();
	Sub ownSub = new Sub("own", 0, 0, 0, new Point2D.Double(0, 0));
	Benel.setOwn(ownSub);
    }




    public static boolean ownExists() {
	return own != null;
    }

    /* Need scrollable, circular map so naval objects can't go off
       the map.  If do, will prob get null pointer exception now.
    */
    void loadHtFld(String image) throws IOException {
	int x = 0;
	int y = 0;
	int n = 0;
	int red = 0;
	int blue = 0;
	int green = 0;

	/* Using a buffered stream and URL to allow the class loader
	   to access files that aren't classes.  The InputStream,
	   BufferedInputStream, and DataInputStream classes
	   leave the incoming bytes as bytes. DataInputStreams operate
	   similarly to Randdom Access Streams. */
	InputStream htFldIn = Benel.class.getResourceAsStream(image);
	BufferedInputStream htFldBuff = new BufferedInputStream(htFldIn);
	DataInputStream htFldData = new DataInputStream(htFldBuff);
	System.out.println("File successfully loaded.");
	htFldData.skipBytes(54); //seeks to end of header
	for (y = 239; y >= 0; y--) {
	    n = 320 * y;
	    for (x = 3 * 320 * y; x < (3 * 320 * (y + 1)); x += 3) {
		blue = htFldData.read();
		green = htFldData.read();
		red = htFldData.read();
		htFldRGB[x] = red;
		htFldRGB[x + 1] = green;
		htFldRGB[x + 2] = blue;
		htFld[n++] = blue;
	    }
	}
	System.out.println("File successfully read.");
    }

    void buildConstraints(GridBagConstraints gbc, int gx, int gy, int gw, int gh, int wx, int wy) {
	gbc.gridx = gx;
	gbc.gridy = gy;
	gbc.gridwidth = gw;
	gbc.gridheight = gh;
	gbc.weightx = wx;
	gbc.weighty = wy;
    }







    public void stateChanged(ChangeEvent evt) {
	if (evt.getSource() instanceof JSlider) {
	    setDepth2Disp(getDepth() + "");
	    setSpeed2Disp(getSpeed() + "");
	    setBearing2Disp(getBearing() + "");
	}
    }

    /* UI accessor methods: JSliders
     */
    public int getDepth() {
	return depth.getValue();
    }

    public int getSpeed() {
	return speed.getValue();
    }

    public int getBearing() {
	return bearing.getValue();
    }

    /* UI mutator methods: JSlider text fields
     */

    public void setDepth2Disp(String s) {
	depth2Disp.setText(s);
    }

    public void setSpeed2Disp(String s) {
	speed2Disp.setText(s);
    }
    
    public void setBearing2Disp(String s) {
	bearing2Disp.setText(s);
    }





    public void focusGained(FocusEvent evt) { }

    public void focusLost(FocusEvent evt) {
	setDepth(Integer.parseInt(getDepth2Disp()));
	setSpeed(Integer.parseInt(getSpeed2Disp()));
	setBearing(Integer.parseInt(getBearing2Disp()));
    }

    /* UI accessors: JSliders' text fields
     */

    public String getDepth2Disp() {
	return depth2Disp.getText();
    }

    public String getSpeed2Disp() {
	return speed2Disp.getText();
    }

    public String getBearing2Disp() {
	return bearing2Disp.getText();
    }

    /* UI mutators: JSliders
     */
    public void setDepth(int i) {
	depth.setValue(i);
    }

    public void setSpeed(int i) {
	speed.setValue(i);
    }

    public void setBearing(int i) {
	bearing.setValue(i);
    }





    public void actionPerformed(ActionEvent evt) {
	// rem that the source is not the ActionListener, 
	// but what causes the evt
	Object source = evt.getSource();

	if (source == move) {
	    own.setIntendedDepth((double)getDepth());
	    own.setIntendedSpeed((double)getSpeed());
	    own.setIntendedBearing((double)getBearing());
	} else if (source == timer) {
	    if (ownExists())
		updateDisps(own.getCoord(), own.getDepth(), own.getSpeed(), own.getBearing(), own.getSeaDepth());
	}
    }




    /* UI mutators: Current stats display
     */
    public void setPositionDisp(String x, String y) {
	positionDisp.setText("(" + x + ", " + y + ")");
    }
	
    public void setDepthDisp(String s) {
	depthDisp.setText(s);
    }
    
    public void setSpeedDisp(String s) {
	speedDisp.setText(s);
    }
    
    public void setBearingDisp(String s) {
	bearingDisp.setText(s);
    }
    
    public void setSeaDepthDisp(String s) {
	seaDepthDisp.setText(s);
    }

	

    /* Updates all the displays for the user's sub stats.
       Apply NumberFormat here so don't have to repeat
       with each display mutator, and can allot the mutators
       the simple function of only setting the display to a 
       given value.
     */
    public void updateDisps(Point2D.Double coord, double depth, double speed, double bearing, double seaDepth) {
	NumberFormat formatter = NumberFormat.getNumberInstance();

	formatter.setMaximumFractionDigits(2);
	setPositionDisp(formatter.format(coord.x), formatter.format(coord.y));
	setDepthDisp(formatter.format(depth));
	setSpeedDisp(formatter.format(speed));
	setBearingDisp(formatter.format(bearing));
	setSeaDepthDisp(formatter.format(seaDepth));
    }
}






/* Fully exit program when close main window.
 */
class ExitWindow extends WindowAdapter {
    public void windowClosing(WindowEvent e) {
	System.exit(0);
    }
}







/* Copyright notice:
 * This program is free software; you can redistribuet it and/or
 * modify it under the terms of the GNU General Public Licencse
 * as published by the Free Softward Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
*/

/* VerHist
 * Lessons:
 -let GridBag create its own proportions: set all weights to 100, only
 adjusting if necessary
 -some cases where necessary: images seem to need BOTH fill or will readily
 give up space; use BOTH fill any time want to fill up remaining space,
 but note that some comps will dominate others when multiple comps
 use this setting along with all weight 100--eg JTextArea dominated
 image; use weight 0 to keep
 at default size--may need to 
 fill in one or both directions to stretch to frame's sides; make sure
 that comp doesn't stretch beyond frame, or comp may disappear
 entirely--eg JTextArea with 40 rows in small frame never showed up,
 and other comps took over the space
 -in short: weight 100 means can resize to fill frame, 
 while 0 means keep default; 
 BOTH fill means will go out in both directions, 
 but weight still seems to have precedence;
 need BOTH fill and weight 100 of at least one comp
 in ea dir to stretch to frame's sides;
 -practically: all BOTH fill; weight 100 for images; weight 100 for
 any comp that want to stretch, otherwise weight 0; some comps
 at weight 100 may dominate others; best to only give one type of comp 
 weight 100
 in a given row or column--stretch rest of comps by BOTH fill;
 only then play with weights, varying corresponding weight between
 0 and 100 for only one comp in given row or column; think in terms
 of each cell dragging out or pushing into its whole row or column;
 SO: BOTH fill, weight 0, then play one comp at a time
 -use Timer to utilize event-dispatching thread rather than creating
 a new one with Runnable
 -if sending in an array parameter, must specify array in signature
 -making array doesn't make the objects in the array; must specify "new"
 for each array element or create via array initializer

 * ToDo:
 */
