/** Panel that houses all the weapons delivery controls, including
    torpedo coordinates and speed and Harpoon targetting.
*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

class FireControlPanel extends JPanel {
    private Timer timer;
    private ScopePanel scope = new ScopePanel();
    private ButtonGroup tubes = new ButtonGroup();
    // may combine to 1 group since only have one set of selector displays
    private ButtonGroup torpedoTubeGroup = new ButtonGroup();
    // creates array to hold 4 JRadioButtons; doesn't create the actual objects
    private JRadioButton[] torpedoTubes = new JRadioButton[4]; // torpedo tubes
    private ButtonGroup vlsGroup = new ButtonGroup();
    private JRadioButton[] vlsTubes = new JRadioButton[12]; // vertical launch tubes
    private JComboBox[] torpedoTubeSelectors = new JComboBox[4];
    private String[] torpedoTubeWeapons = { "ADCAP", "Harpoon", "Captor", "AMM" };
    private JComboBox[] vlsTubeSelectors = new JComboBox[12];
    private String[] vlsTubeWeapons = { "TLAM", "UUV", "UAV" };
    private JTextField[] torpedoStatus = new JTextField[4];
    private JTextField[] vlsStatus = new JTextField[12];
    private JLabel statusLbl = new JLabel("Status:");
    private JTextArea status = new JTextArea(10, 15);
    private GridBagLayout gridBag = new GridBagLayout();
    private GridBagConstraints constraints = new GridBagConstraints();

    FireControlPanel() {
	setLayout(gridBag);

	// scope panel; may eventually replace with customized map
	buildConstraints(constraints, 0, 0, 1, 12, 100, 100);
	constraints.fill = GridBagConstraints.BOTH;
	constraints.anchor = GridBagConstraints.WEST;
	gridBag.setConstraints(scope, constraints);
	add(scope);
	
	// make and display torpedo and vls tube selectors
	makeButtonGroup(tubes, torpedoTubes, "Tor ");
	addComps(torpedoTubes, 4, 1, 0, 3);
	makeButtonGroup(tubes, vlsTubes, "VLS ");
	addComps(vlsTubes, 4, 1, 3, 3);
	torpedoTubes[0].setSelected(true);
	
	// make and display torpedo and vls tube weapons selectors
	makeComboBoxes(torpedoTubeSelectors, torpedoTubeWeapons);
	addComps(torpedoTubeSelectors, 4, 1, 1, 3);
	makeComboBoxes(vlsTubeSelectors, vlsTubeWeapons);
	addComps(vlsTubeSelectors, 4, 1, 4, 3);
	
	// make and display torpedo and vls tube status indicators
	makeStatusIndicators(torpedoStatus);
	addComps(torpedoStatus, 4, 1, 2, 3);
	makeStatusIndicators(vlsStatus);
	addComps(vlsStatus, 4, 1, 5, 3);

	// display status lablel and text area
	buildConstraints(constraints, 5, 0, 1, 1, 0, 0);
	constraints.fill = GridBagConstraints.BOTH;
	constraints.anchor = GridBagConstraints.WEST;
	gridBag.setConstraints(statusLbl, constraints);
	add(statusLbl);
	status.setLineWrap(true);
	status.setWrapStyleWord(true);
	status.setEditable(false);
	buildConstraints(constraints, 5, 1, 1, 6, 0, 100);
	constraints.fill = GridBagConstraints.BOTH;
	constraints.anchor = GridBagConstraints.WEST;
	gridBag.setConstraints(status, constraints);
	add(status);
    }


    void buildConstraints(GridBagConstraints gbc, int gx, int gy, int gw, int gh, int wx, int wy) {
	gbc.gridx = gx;
	gbc.gridy = gy;
	gbc.gridwidth = gw;
	gbc.gridheight = gh;
	gbc.weightx = wx;
	gbc.weighty = wy;
    }

    void makeStatusIndicators(JTextField[] fields) {
	int n;
	for (n = 0; n < fields.length; n++) {
	    fields[n] = new JTextField("Loaded", 6);
	    fields[n].setEditable(false);
	    fields[n].setBackground(Color.green);
	    fields[n].setHorizontalAlignment(JTextField.CENTER);
	}
    }

    void makeComboBoxes(JComboBox[] boxes, String[] items) {
	int n;
	for (n = 0; n < boxes.length; n++) {
	    boxes[n] = new JComboBox(items);
	}
    }

    void makeButtonGroup(ButtonGroup group, JRadioButton[] tubes, String tubeText) {
	int n;
	for (n = 0; n < tubes.length; n++) {
	    tubes[n] = new JRadioButton(tubeText + (n + 1));
	    group.add(tubes[n]);
	}
	tubes[0].setSelected(true);
    }

    /** Dynamically adds buttons to the GridBag layout.
	@param comps array of even number of buttons; must be
	divisible by <code>columns</code>
	@param columns number of columns
	@param firstColumn first column number in layout, starting with 0
	@param firstRow first row number in layout, starting with 0
	@param rowIncrement number of rows to increment
    */
    void addComps(JComponent[] comps, int columns, int firstColumn, int firstRow, int rowIncrement) {
	int n;
	int m;
	for (n = 0; n < (comps.length / columns); n++) {
	    for (m = 0; m < columns; m++) {
		buildConstraints(constraints, m + firstColumn, rowIncrement * n + firstRow, 1, 1, 0, 100);
		constraints.fill = GridBagConstraints.BOTH;
		constraints.anchor = GridBagConstraints.WEST;
		gridBag.setConstraints(comps[n * columns + m], constraints);
		add(comps[n * columns + m]);
	    }
	}
    }
}
