
import java.awt.*;
import javax.swing.*;
import java.awt.Image;
import java.net.*;
import java.awt.event.*;
import java.awt.geom.*;

/* Creates the map screen that draws the sea chart as well as the subs
   and other naval objects in it.

   -origin refers to the center position on the component
   -coords refer to the objects' position relative to the origin
*/
class ScopePanel extends JPanel implements ActionListener {
    private Timer timer;
    public Image mapPic;
    private Color ownColor;
    private Color friendlyColor;
    private Color adversaryColor;
    private int compWidth;
    private int compHeight;
    private int picWidth;
    private int picHeight;
    private int xPic;
    private int yPic;
    private int n = 0;

    // Should likely create new JPanel()
    ScopePanel() {
	super();
	/* Using a URL and getResource() to allow the class loader
	   to access non-class files. */
	String mapSrc = "htFld.jpg";

	URL mapSrcURL = ScopePanel.class.getResource(mapSrc);
	Toolkit kit = Toolkit.getDefaultToolkit();
	mapPic = kit.getImage(mapSrcURL);

	timer = new Timer(500, this);
	timer.start();	
    }



    /* Accessors
     */
    int getMapWidth() {
	return picWidth;
    }

    int getMapHeight() {
	return picHeight;
    }





    /** Alternates between color vehicle display settings.
	<code>darkGray</code> is for the user's submarine,
	<code>gray</code> is for all other friendly submarines, and
	<code>red</code> is for all adversary submarines.
	The alternate color for each vehicle is white
    */
    public void actionPerformed(ActionEvent evt) {
	if (n == 0) {
	    ownColor = Color.darkGray;
	    friendlyColor = Color.gray;
	    adversaryColor = Color.red;
	    repaint();
	    n = 1;
	} else {
	    ownColor = Color.white;
	    friendlyColor = Color.white;
	    adversaryColor = Color.white;
	    repaint();
	    n = 0;
	}
    }

    /* May need to make colors and drawings for neutral objects, too.
     */
    public void paintComponent(Graphics screen) {

	compWidth = getSize().width;
	compHeight = getSize().height;
	picWidth = mapPic.getWidth(this);
	picHeight = mapPic.getHeight(this);
	xPic = (compWidth - picWidth) / 2;
	yPic = (compHeight - picHeight) / 2;
	Benel.setOrigin(compWidth / 2, compHeight / 2);

	/* Merely ensures that bkgrd doesn't have a negative position. */
	if (xPic < 0)
	    xPic = 0;
	if (yPic < 0)
	    yPic = 0;

	Graphics2D comp2D = (Graphics2D)screen;
	// Draw bkgrd
	comp2D.setColor(Color.darkGray);
	comp2D.fillRect(0, 0, compWidth, compHeight);
	//	System.out.println(compWidth + ", " + compHeight + "; " + picWidth + ", " + picHeight + "; " + xPic + ", " + yPic);
	// Draw map
	comp2D.drawImage(mapPic, xPic, yPic, this);


	// Draw naval objects
	Point2D.Double origin = new Point2D.Double();
	Point2D.Double coord = new Point2D.Double();

	// own sub
	comp2D.setPaint(ownColor);
	origin = Benel.getOrigin();
	Ellipse2D.Double objPic;
	if (Benel.ownExists()) {
	    coord = Benel.getOwnCoord();
	    objPic = makeCircle(origin, coord, 3);
	    comp2D.fill(objPic);
	}

	int n;
	for (n = 0; n < Benel.getNumOthers(); n++) {
	    coord = Benel.getOthersCoord(n);
	    objPic = makeCircle(origin, coord, 3);
	    // same side sub, not own
	    if (Benel.getOthersOwnership(n).equals("friendly"))
		comp2D.setPaint(friendlyColor);
	    // adversary sub
	    else comp2D.setPaint(adversaryColor);
	    comp2D.fill(objPic);
	}
	
	for (n = 0; n < Benel.getNumTorpedoes(); n++) {
	    coord = Benel.getTorpedoCoord(n);
	    objPic = makeCircle(origin, coord, 1);
	    // own torpedo
	    if (Benel.getTorpedoesOwnership(n).equals("own"))
		comp2D.setPaint(ownColor);
	    // friendly torpedo, not own
	    else if (Benel.getTorpedoesOwnership(n).equals("friendly"))
		comp2D.setPaint(friendlyColor);
	    // adversary torpedo
	    else comp2D.setPaint(adversaryColor);
	    comp2D.fill(objPic);
	}
    }

    Ellipse2D.Double makeCircle(Point2D.Double origin, Point2D.Double coord, double d) {
	return  new Ellipse2D.Double(origin.x + coord.x, origin.y - coord.y, d, d);
    }
}
